/*
	Copyright (C) 2009 Karl Zeilhofer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * Graphical User Interface for openOCD
 * with emphasis on Programming
 *

 Version History:
 * 2009 07 01: 0.1 first usable release
 *              hard coded host and port on telnet
                hard coded register addresses
                hard coded flash size
   2009 07 02: 0.1.1 more comftable
                save recent directory in recentdir.dat

   2010 03 06: 0.1.2 some nice to have features
		editable hostname and port for telnet
		transformed resetbutton to connectionreset
		added button for openocd-command 'reset'
		improved connectbutton to disconnect
 */
#define SAM7_VERSION "0.1.2"

#include "mainwidget.h"
#include "ui_mainwidget.h"
#include <QScrollBar>
#include <QFileDialog>
#include <QByteArray>
#include <QRect>

//using namespace standard

MainWidget::MainWidget(QWidget *parent)
    : QWidget(parent), ui(new Ui::MainWidget)
{
    ui->setupUi(this);

    setGeometry(QRect(100,100,800,480));
    setWindowTitle(QString("SAM7 openOCD GUI v") + SAM7_VERSION);

    tn = new QtTelnet(this);

// control buttons
    connect(ui->pushButtonOocdConnect, SIGNAL(clicked()),
            this, SLOT(connectToServer()));
    connect(tn, SIGNAL(message(QString)),
            this, SLOT(telnetMessage(QString)));

    connect(ui->pushButtonOocdReset, SIGNAL(clicked()),
            this, SLOT(resetOocd()));
    connect(ui->lineEditInput, SIGNAL(returnPressed()),
            this, SLOT(telnetData()));

// ram
    connect(ui->pushButtonRamFile, SIGNAL(clicked()),
            this, SLOT(ramFileSelect()));
    connect(ui->pushButtonRamLoad, SIGNAL(clicked()),
            this, SLOT(ramLoad()));
// flash
    connect(ui->pushButtonFlashFile, SIGNAL(clicked()),
            this, SLOT(flashFileSelect()));
    connect(ui->pushButtonFlashLoad, SIGNAL(clicked()),
            this, SLOT(flashLoad()));
// command buttons
    connect(ui->pushButtonSoftReset, SIGNAL(clicked()),
            this, SLOT(softReset()));
    connect(ui->pushButtonReset, SIGNAL(clicked()),
            this, SLOT(reset()));
    connect(ui->pushButtonResume, SIGNAL(clicked()),
            this, SLOT(resume()));
    connect(ui->pushButtonHalt, SIGNAL(clicked()),
            this, SLOT(halt()));
    connect(ui->pushButtonPoll, SIGNAL(clicked()),
            this, SLOT(poll()));
    connect(ui->pushButtonShowMem, SIGNAL(clicked()),
            this, SLOT(showMem()));
    connect(ui->pushButtonEraseFlash, SIGNAL(clicked()),
            this, SLOT(eraseFlash()));
    connect(ui->pushButtonRemap, SIGNAL(clicked()),
            this, SLOT(remap()));
    connect(ui->pushButtonPeriphReset, SIGNAL(clicked()),
            this, SLOT(periphReset()));
    connect(ui->pushButtonCpuReset, SIGNAL(clicked()),
            this, SLOT(cpuReset()));

    QFile file(DIR_FILE_NAME);
    if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QTextStream in(&file);
        recentDir = in.readLine(256);
    }

}

MainWidget::~MainWidget()
{
    QFile file(DIR_FILE_NAME);
    if (file.open(QIODevice::Truncate | QIODevice::WriteOnly | QIODevice::Text))
    {
        QTextStream out(&file);
        out << recentDir;
    }

    delete ui;
}



// private Slots:

// control buttons:
void MainWidget::connectToServer()
{
    if (ui->pushButtonOocdConnect->text() == "Connect")
    {
	tn->connectToHost(ui->lineEditHost->text(), ui->lineEditPort->text().toInt());
	ui->textEditOutput->append(": Connected to " + ui->lineEditHost->text() + ":" + ui->lineEditPort->text());
	ui->pushButtonOocdConnect->setText("Disconnect");
    }
    else
    {
	tn->close();
	ui->textEditOutput->append(": Connection closed");
	ui->pushButtonOocdConnect->setText("Connect");
    }
}

void MainWidget::resetOocd()
{
    if (ui->pushButtonOocdConnect->text() == "Disconnect")
    {
	tn->close();
	tn->connectToHost(ui->lineEditHost->text(), ui->lineEditPort->text().toInt());
	ui->textEditOutput->append(": Reset Connection");
    }
    else
	ui->textEditOutput->append(": Not connected");
}


void MainWidget::telnetMessage(const QString &msg) // receive output
{
    ui->textEditOutput->append(stripCR(msg));
    QScrollBar *s = ui->textEditOutput->verticalScrollBar();
    s->setValue(s->maximum());
}

void MainWidget::telnetData() // send command
{
    tn->sendData(ui->lineEditInput->text());
    ui->lineEditInput->clear();
}


void MainWidget::ramFileSelect()
{
    QFileDialog fDlg(this, "Select RAM Image", recentDir, "*.bin *.BIN");

    if(fDlg.exec())
    {
        ui->lineEditRam->setText(fDlg.selectedFiles().at(0));
        recentDir = fDlg.directory().absolutePath();
    }
}

void MainWidget::ramLoad() // download image to RAM
{
    tn->sendData("soft_reset_halt");
    tn->sendData("load_image " + ui->lineEditRam->text() + " 0x200000");
}

void MainWidget::flashFileSelect()
{
    QFileDialog fDlg(this, "Select FLASH Image", recentDir, "*.bin *.BIN");

    if(fDlg.exec())
    {
        ui->lineEditFlash->setText(fDlg.selectedFiles().at(0));
        recentDir = fDlg.directory().absolutePath();
    }
}

void MainWidget::flashLoad() // download image to FLASH
{
    tn->sendData("soft_reset_halt");
    tn->sendData("flash write_image erase " + ui->lineEditFlash->text() + " 0x100000");
}



// command buttons:
void MainWidget::softReset()
{
    tn->sendData("soft_reset_halt");
}

void MainWidget::reset()
{
    tn->sendData("reset");
}

void MainWidget::halt()
{
    tn->sendData("halt");
}

void MainWidget::resume()
{
    tn->sendData("resume");
}

void MainWidget::poll()
{
    tn->sendData("poll");
}

void MainWidget::eraseFlash()
{
    tn->sendData("soft_reset_halt");
    tn->sendData("flash erase_address 0x100000 0x10000");
}

//
void MainWidget::showMem()
{
    tn->sendData("mdw 0x000000 0x08"); // mapped memory
    tn->sendData("mdw 0x100000 0x08"); // flash
    tn->sendData("mdw 0x200000 0x08"); // sram
}

void MainWidget::remap()
{
    tn->sendData("mww 0xffffff00 0x00000001");
}

void MainWidget::periphReset()
{
    tn->sendData("mww 0xfffffd00 0xa5000004");
}

void MainWidget::cpuReset()
{
    tn->sendData("mww 0xfffffd00 0xa5000001");
}






// private Funktions:
QString MainWidget::stripCR(const QString &msg)
{
    QString nmsg(msg);
    nmsg.remove('\r');
    //nmsg.remove('\n\r\n\r');
    nmsg.remove(QRegExp("\033\\[[0-9;]*[A-Za-z]")); // Also remove terminal control codes
    return nmsg.isEmpty() ? NULL : nmsg;
}

void MainWidget::removeEmptyLines()
{

}



